#ifndef __Srclist_h__
#define __Srclist_h__

//----------------------------------------------------------------
// srclist.h
//
// Definition of Sourcelist class
//----------------------------------------------------------------
#include <math.h>
#include "Dal.h"
#include "param.h"
#include "Error.h"
#include "Image.h"

#ifndef MAX_FLOAT
#define MAX_FLOAT ((float)3.40282346638528860e+38)
#endif
#ifndef MAX_INT
#define MAX_INT 32768
#endif
#ifndef M_SQRT2
#define M_SQRT2  1.414213562
#endif


#define NMAX_SRCS 100000

float correlationError( float scale, float sigma, float bgd, float nsrc);
float sigmaError( float sqrtratio, float corrRatioError, float scale1,
		  float scale2, float sigma);
float correlationCovariance(float scale1, float scale2, float sigma, float bgd,
			    float nsrc);
float correlationRatioError(float corr1, float dcorr1, 
			    float corr2, float dcorr2, float covariance);



class Sourcelist{
 public:
  Sourcelist( unsigned int n=NMAX_SRCS);
  ~ Sourcelist( );

  void init( void );
  Status allocate(int n);
  Status deallocate( void );
  Status copy( const Sourcelist &srclist);
  Status reducelist( void );
  Status smallfirst( void );
  Status calcCoordinates(const Image & im);
  Status calcScale();
  Status calcCnterr();
  void copyKeywords(const Image &im);
  Status statistics();
  Status writeset( const string &setname );
  Status readset( const string &setname );
  unsigned int ndetect, nmax;

  unsigned int *ipix, *jpix;
  float *xima, *yima, *xerr, *yerr, *xycorr;
  float *scale, *corr, *corr0, *correrr, *crcorr, *sigma, *sigmaErr;
  float *cnts, *cntserr, *cntrate, *cntraterr;
  float *exposure, *background;
  bool *ignore;


 private:
  int nsrcs;
  float *radec_err;
  double *ra, *dec, *lii, *bii;

  string instrument, telescope, detnam, datamode, filter, obs_mode;
  string obs_id, exp_id, content, origin, object;
  string radecsys, observer, obsdate, enddate;
  float ra_pnt, dec_pnt, pa_pnt, equinox, pexposure;

  Status calcProperties(unsigned int i, unsigned int jmin,
			float *sigma2, float *cnts2, 
		        float *corr2err, 
			float *cnts2err,
			float *sigma2err);
};

#endif
